@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2011, Vitalii Vanovschi
+# Copyright (c) 2005-2012, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -44,15 +44,12 @@ import pptransport
 import ppauto
 import ppcommon
 
-copyright = "Copyright (c) 2005-2011 Vitalii Vanovschi. All rights reserved"
-version = "1.6.1"
+copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
+version = "1.6.2"
 
 # Reconnect persistent rworkers in seconds.
 RECONNECT_WAIT_TIME = 5
 
-# Set timeout for socket operations in seconds.
-SOCKET_TIMEOUT = 5
-
 # If set to true prints out the exceptions which are expected.
 SHOW_EXPECTED_EXCEPTIONS = False
 
@@ -145,12 +142,11 @@ class _Worker(object):
     def start(self):
         """Starts local worker"""
         if _USE_SUBPROCESS:
-            proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, \
-                    stdout=subprocess.PIPE, stderr=subprocess.PIPE, \
-                    shell=False)
+            proc = subprocess.Popen(self.command, stdin=subprocess.PIPE,
+                    stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             self.t = pptransport.CPipeTransport(proc.stdout, proc.stdin)
         else:
-            self.t = pptransport.CPipeTransport(\
+            self.t = pptransport.CPipeTransport(
                     *popen2.popen3(self.command)[:2])
 
         self.pid = int(self.t.receive())
@@ -181,7 +177,7 @@ class _RWorker(pptransport.CSocketTransport):
     """Remote worker class
     """
 
-    def __init__(self, host, port, secret, server,  message=None, persistent=True):
+    def __init__(self, host, port, secret, server,  message, persistent, socket_timeout):
         """Initializes remote worker"""
         self.server = server
         self.persistent = persistent
@@ -192,6 +188,7 @@ class _RWorker(pptransport.CSocketTransport):
         self.id = host + ":" + str(port)
         self.server.logger.debug("Creating Rworker id=%s persistent=%s"
                 % (self.id, persistent))
+        self.socket_timeout = socket_timeout
         self.connect(message)
 
     def __del__(self):
@@ -202,7 +199,7 @@ class _RWorker(pptransport.CSocketTransport):
         """Connects to a remote server"""
         while True and not self.server._exiting:
             try:
-                pptransport.SocketTransport.__init__(self)                
+                pptransport.SocketTransport.__init__(self, None, self.socket_timeout) 
                 self._connect(self.host, self.port)                
                 if not self.authenticate(self.secret):
                     self.server.logger.error("Authentication failed for host=%s, port=%s"
@@ -283,7 +280,7 @@ class Server(object):
     default_secret = "epo20pdosl;dksldkmm"
 
     def __init__(self, ncpus="autodetect", ppservers=(), secret=None,
-            restart=False, proto=2):
+            restart=False, proto=2, socket_timeout=3600):
         """Creates Server instance
 
            ncpus - the number of worker processes to start on the local
@@ -296,6 +293,10 @@ class Server(object):
                    custom passphrase for all network connections.
            restart - whether to restart worker process after each task completion
            proto - protocol number for pickle module
+           socket_timeout - socket timeout in seconds which is also the maximum 
+                   time a remote job could be executed. Increase this value
+                   if you have long running jobs or decrease if connectivity
+                   to remote ppservers is often lost.
 
            With ncpus = 1 all tasks are executed consequently
            For the best performance either use the default "autodetect" value
@@ -327,6 +328,7 @@ class Server(object):
         self.__active_rworkers_list_lock = threading.Lock()
         self.__restart_on_free = restart
         self.__pickle_proto = proto
+        self.__connect_locks = {}
 
         # add local directory and sys.path to PYTHONPATH
         pythondirs = [os.getcwd()] + sys.path
@@ -341,6 +343,7 @@ class Server(object):
 
         self.ppservers = []
         self.auto_ppservers = []
+        self.socket_timeout = socket_timeout
 
         for ppserver in ppservers:
             ppserver = ppserver.split(":")
@@ -569,20 +572,24 @@ class Server(object):
 
     def connect1(self, host, port, persistent=True):
         """Conects to a remote ppserver specified by host and port"""        
+        hostid = host+":"+str(port)
+        lock = self.__connect_locks.setdefault(hostid, threading.Lock())
+        lock.acquire()
         try:
-            rworker = _RWorker(host, port, self.secret, self, "STAT", persistent)
+            if hostid in self.autopp_list:
+                return
+            rworker = _RWorker(host, port, self.secret, self, "STAT", persistent, self.socket_timeout)
             ncpus = int(rworker.receive())
-            hostid = host+":"+str(port)
             self.__stats[hostid] = _Statistics(ncpus, rworker)
 
             for x in range(ncpus):
-                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent)
+                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent, self.socket_timeout)
                 self.__update_active_rworkers(rworker.id, 1)
                 # append is atomic - no need to lock self.__rworkers
                 self.__rworkers.append(rworker)
             #creating reserved rworkers
             for x in range(ncpus):
-                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent)
+                rworker = _RWorker(host, port, self.secret, self, "EXEC", persistent, self.socket_timeout)
                 self.__update_active_rworkers(rworker.id, 1)
                 self.__rworkers_reserved.append(rworker)
             self.logger.debug("Connected to ppserver (host=%s, port=%i) \
@@ -591,6 +598,8 @@ class Server(object):
         except:
             if SHOW_EXPECTED_EXCEPTIONS:
                 self.logger.debug("Exception in connect1 method (possibly expected)", exc_info=True)
+        finally:
+            lock.release()
 
     def __connect(self):
         """Connects to all remote ppservers"""
@@ -712,14 +721,13 @@ class Server(object):
             worker.t.csend(sfunc)
             worker.t.send(sargs)
             sresult = worker.t.receive()
+            job.finalize(sresult)
         except:
             if self._exiting:
                 return
             if SHOW_EXPECTED_EXCEPTIONS:
                 self.logger.debug("Exception in _run_local (possibly expected)", exc_info=True)
 
-        job.finalize(sresult)
-
         # remove the job from the waiting list
         if self.__waittasks:
             self.__waittasks_lock.acquire()
@@ -743,6 +751,7 @@ class Server(object):
             rworker.send(sargs)
             sresult = rworker.receive()
             rworker.is_free = True
+            job.finalize(sresult)
         except:
             self.logger.debug("Task %i failed due to broken network " \
                     "connection - rescheduling",  job.tid)
@@ -754,7 +763,6 @@ class Server(object):
                 self.__scheduler()
             return
 
-        job.finalize(sresult)
 
         # remove the job from the waiting list
         if self.__waittasks:
