@@ -1,5 +1,5 @@
 # Parallel Python Software: http://www.parallelpython.com
-# Copyright (c) 2005-2011, Vitalii Vanovschi
+# Copyright (c) 2005-2012, Vitalii Vanovschi
 # All rights reserved.
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -32,13 +32,13 @@ forums
 
 import threading
 
-copyright = "Copyright (c) 2005-2011 Vitalii Vanovschi. All rights reserved"
-version = "1.6.1"
+copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
+version = "1.6.2"
 
 def start_thread(name,  target,  args=(),  kwargs={},  daemon=True):
     """Starts a thread"""
     thread = threading.Thread(name=name,  target=target, args=args,  kwargs=kwargs)
-    thread.daemon = True
+    thread.daemon = daemon
     thread.start()
     return thread
 
